package org.jarbframework.populator;

import static org.jarbframework.utils.Asserts.notNull;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.lang3.builder.ToStringBuilder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Database populator that can skip exceptions from a delegate populator.
 * This populator can be used to prevent exceptions from being thrown.
 * 
 * @author Jeroen van Schagen
 * @since 17-06-2011
 */
public class FailSafeDatabasePopulator implements DatabasePopulator {
    private final Logger logger = LoggerFactory.getLogger(FailSafeDatabasePopulator.class);

    /** Populator that might throw an exception. **/
    private final DatabasePopulator populator;

    /** Classes of exceptions that should be skipped. **/
    private Set<Class<? extends Exception>> skippingExceptionClasses;
    /** Determines if occured exceptions should be logged. **/
    private boolean logError = true;

    /**
     * Construct a new {@link FailSafeDatabasePopulator}.
     * @param populator the populator being wrapped
     */
    public FailSafeDatabasePopulator(DatabasePopulator populator) {
        this(populator, Exception.class);
    }

    /**
     * Construct a new {@link FailSafeDatabasePopulator}.
     * @param populator the populator being wrapped
     * @param skippingExceptionClass exception class being skipped
     */
    public FailSafeDatabasePopulator(DatabasePopulator populator, Class<? extends Exception> skippingExceptionClass) {
        this.populator = notNull(populator, "Populator cannot be null");
        skippingExceptionClasses = new HashSet<Class<? extends Exception>>();
        skippingExceptionClasses.add(skippingExceptionClass);
    }

    /**
     * Configure the exception classes that should be skipped.
     * @param skippingExceptionClasses exception classes being skipped
     */
    public void setSkippingExceptionClasses(Set<Class<? extends Exception>> skippingExceptionClasses) {
        this.skippingExceptionClasses = skippingExceptionClasses;
    }

    /**
     * Configure whether exceptions should be logged or not.
     * @param logError {@code true} if exceptions should be logged, else {@code false}
     */
    public void setLogError(boolean logError) {
        this.logError = logError;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void populate() throws Exception {
        try {
            populator.populate();
        } catch (Exception e) {
            if (isSkipableException(e)) {
                logIfNeeded(e);
            } else {
                throw e;
            }
        }
    }

    /**
     * Determine if an exception is skipable. Whenever it is skipable,
     * we will prevent the exception from being thrown.
     * @param e exception that was initially thrown
     * @return {@code true} if it should be skipped, else {@code false}
     */
    private boolean isSkipableException(Exception e) {
        for (Class<? extends Exception> skippingExceptionClass : skippingExceptionClasses) {
            if (skippingExceptionClass.isAssignableFrom(e.getClass())) {
                return true;
            }
        }
        return false;
    }

    /**
     * Log the exception whenever desired.
     * @param e exception that should be logged
     */
    private void logIfNeeded(Exception e) {
        if (logError) {
            logger.info("An error occured while executing database populator (" + populator + ").", e);
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        return ToStringBuilder.reflectionToString(this);
    }

}
